import enum

class asyauthProtocol(enum.Enum):
	NONE = 'NONE'
	SIMPLE = 'SIMPLE'
	PLAIN = 'PLAIN'
	SICILY = 'SICILY'
	NTLM = 'NTLM'
	KERBEROS = 'KERBEROS'
	CREDSSP = 'CREDSSP'
	SPNEGO = 'SPNEGO'
	SPNEGOEX = 'SPNEGOEX'
	SSL = 'SSL' # SSL/TLS schannel later

class asyauthSubProtocol(enum.Enum):
	NATIVE = 'NATIVE'
	SSPI = 'SSPI'
	WSNET = 'WSNET'
	SSPIPROXY = 'SSPIPROXY'
	CUSTOM = 'CUSTOM'

class asyauthSecret(enum.Enum):
	NONE = 'NONE'
	PASSWORD = 'PASSWORD'
	PASS = 'PASS'
	PW = 'PW'
	PWB64 = 'PWB64'
	PWPROMPT = 'PWPROMPT'
	PWHEX = 'PWHEX'
	LM = 'LM'
	NT = 'NT'
	RC4 = 'RC4'
	AES = 'AES'
	CCACHE = 'CCACHE'
	CCACHEHEX = 'CCACHEHEX'
	CCACHEB64 = 'CCACHEB64'
	KEYTAB = 'KEYTAB'
	KEYTABHEX = 'KEYTABHEX'
	KEYTABB64 = 'KEYTABB64'
	PFX = 'PFX'
	PFXB64 = 'PFXB64'
	PFXHEX = 'PFXHEX'
	PFXSTR = 'PFXSTR'
	PEM = 'PEM'
	CERTSTORE = 'CERTSTORE'
	KIRBI = 'KIRBI'
	KIRBIHEX = 'KIRBIHEX'
	KIRBIB64 = 'KIRBIB64'
	DES = 'DES'
	TDES = 'TDES'
	DES3 = 'DES3'
	SSHPRIVKEY = 'SSHPRIVKEY'
	SSHPRIVKEYSTR = 'SSHPRIVKEYSTR'
	SSHPRIVKEYB64 = 'SSHPRIVKEYB64'